/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        20.05.2024
 Description:    DMX transmit Routinen
------------------------------------------------------------------------------*/

#include "dmx_transmit.h"

volatile uint8_t dmxUniverse[MAX_CHANNELS];

// ----------------------------------------------------------------------------
// DMX transmission init
void dmx_transmit_start (void){
	
	DDRD |= (1<<PD1 | PD2);
	PORTD |= (1<<PD2);
	
	UBRR0   = (F_CPU / (250000* 16L) - 1);
	UCSR0C|= (1<<USBS0); //USBS0 2 Stop bits
	UCSR0B = (1<<TXEN0)|(1<<TXCIE0); // TXEN0 Transmitter enable / TXCIE0 TX complete interrupt enable
	UDR0    = 0;//start transmitting
}

// ----------------------------------------------------------------------------
// Clear output
void dmx_transmit_clear (void){
	uint8_t count = 0;
	
	for(count=0;count<sizeof(dmxUniverse);count++){
		dmxUniverse[count] = 0;
	}
}

// ----------------------------------------------------------------------------
// DMX transmission
ISR (USART0_TX_vect) {
	static unsigned char  dmxState = BREAK;
	static unsigned short curDmxCh = 0;

	switch(dmxState){	
		case(BREAK):{
			UBRR0 = (F_CPU / (50000 * 16L) - 1);
			UDR0  = 0;								//send break
			dmxState = STARTB;
			return;
		}
			
		case(STARTB):{
			UBRR0 = (F_CPU / (250000 * 16L) - 1);
			UDR0  = 0;								//send start byte
			dmxState = DATA;
			curDmxCh = 0;
			return;
		}
	}
	
	_delay_us(IBG);
	
	UDR0 = dmxUniverse[curDmxCh++];			//send data
	
	if (curDmxCh >= sizeof(dmxUniverse)){
		dmxState = BREAK; 						//new break if all ch sent
	}

}
